import _extends from 'babel-runtime/helpers/extends';
import _objectWithoutProperties from 'babel-runtime/helpers/objectWithoutProperties';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import { Field } from './field';
import { FormContext } from './form-context';
import { FieldProps } from './field-props';

var FormField = function (_React$Component) {
  _inherits(FormField, _React$Component);

  function FormField(props, context) {
    _classCallCheck(this, FormField);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props, context));

    _this.formError = false;

    _this.onInit = function (field) {
      // console.log('onInit', field);
      _this.context.form.onInitField(field);
      _this.onValid(field);
    };

    _this.onFocus = function (field, e) {
      // console.log('onFocus', field);
      _this.context.form.onUpdateField(field);
      var _this$props$onFocus = _this.props.onFocus,
          onFocus = _this$props$onFocus === undefined ? function () {} : _this$props$onFocus;

      onFocus(e, field);
      _this.onValid(field);
    };

    _this.onBlur = function (field, e) {
      // console.log('onBlur', field);
      _this.context.form.onUpdateField(field);
      var _this$props$onBlur = _this.props.onBlur,
          onBlur = _this$props$onBlur === undefined ? function () {} : _this$props$onBlur;

      onBlur(e, field);
      _this.onValid(field);
    };

    _this.onChange = function (field, e) {
      // console.log('onChange', field);
      _this.context.form.onUpdateField(field);
      var _this$props$onChange = _this.props.onChange,
          onChange = _this$props$onChange === undefined ? function () {} : _this$props$onChange;

      onChange(e, field);
      _this.onValid(field);
    };

    _this.onUpdate = function (field) {
      // console.log('onUpdate', field);
      _this.context.form.onUpdateField(field);
      _this.onValid(field);
    };

    _this.onAsyncValid = function (field) {
      // console.log('onAsyncValid', field);
      if (!_this.removed) {
        _this.context.form.onUpdateField(field);
        _this.onValid(field);
      }
    };

    _this.onRemove = function (field) {
      // console.log('onRemove', field);
      _this.removed = true;
      _this.context.form.onRemoveField(field);
    };

    _this.removed = false;
    _this.state = { errorValidators: [] };
    return _this;
  }

  FormField.prototype.componentDidMount = function componentDidMount() {
    var _this2 = this;

    this.subscriberId = this.context.form.validators.subscribe(function (error) {
      if (_this2.formError !== error) {
        _this2.formError = error;
        _this2.onValid(_this2.context.form.fields.getFieldByName(_this2.props.name));
      }
    }, this.props.name);
  };

  FormField.prototype.componentWillUnmount = function componentWillUnmount() {
    this.context.form.validators.removeSubscriber(this.subscriberId);
    this.subscriberId = null;
  };

  FormField.prototype.onValid = function onValid(field) {
    var _props$onValid = this.props.onValid,
        onValid = _props$onValid === undefined ? function () {} : _props$onValid;

    onValid(field.getFieldOptions(), this.formError);
  };

  FormField.prototype.render = function render() {
    var _context$form = this.context.form,
        formValidators = _context$form.formValidators,
        formValidatorsOptions = _context$form.formValidatorsOptions,
        formAsyncValidator = _context$form.formAsyncValidator,
        formAsyncValidatorOptions = _context$form.formAsyncValidatorOptions;

    var _props = this.props,
        name = _props.name,
        _props$validators = _props.validators,
        validators = _props$validators === undefined ? [] : _props$validators,
        _props$validatorsOpti = _props.validatorsOptions,
        validatorsOptions = _props$validatorsOpti === undefined ? {} : _props$validatorsOpti,
        asyncValidator = _props.asyncValidator,
        _props$asyncValidator = _props.asyncValidatorOptions,
        asyncValidatorOptions = _props$asyncValidator === undefined ? {} : _props$asyncValidator,
        other = _objectWithoutProperties(_props, ['name', 'validators', 'validatorsOptions', 'asyncValidator', 'asyncValidatorOptions']);

    return React.createElement(Field, _extends({}, other, {
      name: name,
      validators: [].concat(validators, formValidators[name] || [], this.state.errorValidators),
      validatorsOptions: _extends({}, formValidatorsOptions[name] || {}, validatorsOptions),
      asyncValidator: asyncValidator || formAsyncValidator[name],
      asyncValidatorOptions: _extends({}, formAsyncValidatorOptions[name] || {}, asyncValidatorOptions),
      onInit: this.onInit,
      onChange: this.onChange,
      onFocus: this.onFocus,
      onBlur: this.onBlur,
      onUpdate: this.onUpdate,
      onAsyncValid: this.onAsyncValid,
      onRemove: this.onRemove
    }));
  };

  return FormField;
}(React.Component);

FormField.propTypes = _extends({}, FieldProps, {
  onValid: React.PropTypes.func
});

FormField.contextTypes = {
  form: React.PropTypes.instanceOf(FormContext).isRequired
};

export { FormField };