'use strict';

var _formContext = require('../form-context');

var _fieldAttr = require('../field-attr');

var _validators = require('../validators');

jest.useFakeTimers();

describe('FormContext', function () {
  describe('Fields', function () {
    var context = void 0;
    var fields = void 0;
    beforeEach(function () {
      context = new _formContext.FormContext();
      fields = context.fields;
    });

    it('should have initialize values', function () {
      expect(fields.subscribers).toEqual({});
      expect(fields.fields).toEqual({});
      expect(fields.lastId).toEqual(0);
    });

    it('should add field', function () {
      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.fields.name).toEqual(field);
    });

    it('should get fields', function () {
      var _expect$toEqual;

      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getFields()).toEqual((_expect$toEqual = {}, _expect$toEqual[field.name] = field, _expect$toEqual));
    });

    it('should get field by name', function () {
      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getFieldByName(field.name)).toEqual(field);
    });

    it('should get field options by name', function () {
      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getFieldOptions('wrong-name')).not.toBeDefined();
      expect(fields.getFieldOptions(field.name)).toEqual(field.getFieldOptions());
    });

    it('should get fields options', function () {
      var _expect$toEqual2;

      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getFieldsOptions()).toEqual((_expect$toEqual2 = {}, _expect$toEqual2[field.name] = field.getFieldOptions(), _expect$toEqual2));
    });

    it('should get fields values', function () {
      var _expect$toEqual3;

      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getValues()).toEqual((_expect$toEqual3 = {}, _expect$toEqual3[field.name] = field.value, _expect$toEqual3));
    });

    it('should get valid status', function () {
      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.isValid()).toEqual(true);


      field.setValidators([new _validators.RequiredValidator()]);
      field.validate();
      fields.updateField(field);
      expect(fields.isValid()).toEqual(false);

      field.setValidators();
      field.validate();
      field.pending = true;
      fields.updateField(field);
      expect(fields.isValid()).toEqual(false);
    });

    it('should set fields as used', function () {
      var mockFn = jest.fn();
      var field = new _fieldAttr.FieldAttr({ validate: mockFn }, 'name');
      fields.updateField(field);
      fields.updateFieldsAsUsed();
      expect(mockFn).toHaveBeenCalled();
    });

    it('should reset fields', function (done) {
      var mockFn = jest.fn();
      var field = new _fieldAttr.FieldAttr({ reset: mockFn }, 'name');
      fields.updateField(field);
      fields.reset().then(function () {
        expect(mockFn).toHaveBeenCalled();
        done();
      });
    });

    it('should remove field', function () {
      var field = new _fieldAttr.FieldAttr({}, 'name');
      fields.updateField(field);
      expect(fields.getFieldByName(field.name)).toBeDefined();
      fields.removeField(field.name);
      expect(fields.getFieldByName(field.name)).not.toBeDefined();
    });
  });

  describe('Validators', function () {});
});
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }
}();

;